//3D program in DirectX, arrows control the camera (and partially the cube too)
//L turns lighting on or off
//SPACE makes the cube jump
//W moves the cube forward and S moves it backwards (both in relation to the camera)
//R and F move the cube ON the z-axis and D and G move it on X-axis
//the RFDG controls are in relation to the axis and not the camera

//Includes for Windows stuff and Direct3d 8, Lean and Mean cuts out rarely used stuff
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <D3DX8.h>
#include <dinput.h>

//Instead of adding these to the linked list, we just use this method
#pragma comment(lib,"d3d8.lib")
#pragma comment(lib,"d3dx8.lib")
#pragma comment(lib,"dinput8.lib")
#pragma comment(lib,"dxguid.lib")

//Delcare functions
void FatalError(const char *error_msg);
void ask_fullscreen(void);
LRESULT CALLBACK default_window_proc(HWND hwnd,UINT msg,WPARAM wparam,LPARAM lparam);
void init_window(void);
void kill_window(void);
void init_d3d(void);
void kill_d3d(void);
void init_scene(void);
void kill_scene(void);
void message_pump(void);
D3DFORMAT find_16bit_mode(void);
void render(void);
void NOP(HINSTANCE p_prev_instance,LPSTR p_cmd_line,int p_show);
void keyinput(void);
void load_mesh(LPDIRECT3DDEVICE8 meshdevice, LPSTR meshName);
void draw(float rotx, float roty, float transx, float transy, float transz, LPDIRECT3DTEXTURE8 textName,
		  D3DPRIMITIVETYPE drawType, int startVertice, int numVertice);

//App name for windows, errors, and messages
const char app_name[]="Pyramid Control";

// default screen size
const int app_width=800;
const int app_height=600;
bool app_fullscreen=true;//windowed or full-screen?
HWND app_main_window=NULL;// global handle for main window

// global handle to our instance which is occasionally needed by Windows API
HINSTANCE app_instance;
bool app_done=false; // On true, clean up and exit app

// Main D3D interface, first object you create and last you release
LPDIRECT3D8 app_D3D=NULL;

// Texture variables
LPDIRECT3DTEXTURE8 app_texture=NULL, sky_texture=NULL, wall_texture=NULL, floor_texture=NULL;

LPD3DXBUFFER MaterialBuffer; //for loading mesh/model
LPD3DXMESH mesh; //the variable for our mesh
DWORD NumMaterials; //all this relates to loading in models
D3DMATERIAL8 *MeshMaterials;
LPDIRECT3DTEXTURE8 *MeshTextures;

LPDIRECTINPUT8 app_DirectInput; //pointer to dinput interface
LPDIRECTINPUTDEVICE8 app_Keyboard; //pointer to our keyboard
#define KEYDOWN(name, key) (name[key] & 0x80)

// D3DDevice is the main rendering interface. When you do anything to be displayed on 
//screen, chances are it will be done through this device. 
IDirect3DDevice8 *app_d3d_device=NULL;

//Struct with all info about the vertex we need
struct my_vertex{
    FLOAT x, y, z;   // untransformed position
	FLOAT nx,ny,nz; //the "normal" coordinates
    DWORD color;     // color
	FLOAT tu, tv; //texture coordinates tu = x
};

//Macro for vertex definition. We tell D3D what we are passing when we use the vertex data.
//DIFFUSE is specifying it has color and XYZ is coordinates in model space
//TEX1 specifies that we are using a texture NORMAL dictates the presence of a normal
#define D3D8T_CUSTOMVERTEX (D3DFVF_XYZ|D3DFVF_TEX1|D3DFVF_NORMAL|D3DFVF_DIFFUSE)

//Vertices of the pyramid, with different colors throughout. 0,0,0 is center. 
my_vertex app_vertices[] ={
   {  0.0f,  1.0f, 0.0f, 0.0f,  1.0f, 0.0f, 0xFFFFFFFF, 0.5f, 0.0f }, // top corner
   {  1.0f, -1.0f,-1.0f, 1.0f, -1.0f,-1.0f, 0xFFFFFFFF, 1.0f, 1.0f }, // right, front
   { -1.0f, -1.0f,-1.0f,-1.0f, -1.0f,-1.0f, 0xFFFFFFFF, 0.0f, 1.0f }, // left, front
   { -1.0f, -1.0f, 1.0f,-1.0f, -1.0f, 1.0f, 0xFFFFFFFF, 0.0f, 1.0f }, // back, left
   {  1.0f, -1.0f, 1.0f, 1.0f, -1.0f, 1.0f, 0xFFFFFFFF, 1.0f, 1.0f }, // back, right
   {  1.0f, -1.0f,-1.0f, 1.0f, -1.0f,-1.0f, 0xFFFFFFFF, 1.0f, 1.0f }, // right, front (close the pyramid)

   {  1.0f, -1.0f, -1.0f, 0.0f, -1.0f, 0.0f, 0xFFFFFFFF, 0.0f, 1.0f },  //Bottom face
   {  1.0f, -1.0f,  1.0f, 0.0f, -1.0f, 0.0f, 0xFFFFFFFF, 0.0f, 0.0f },
   { -1.0f, -1.0f,  1.0f, 0.0f, -1.0f, 0.0f, 0xFFFFFFFF, 1.0f, 0.0f },
   { -1.0f, -1.0f,  1.0f, 0.0f, -1.0f, 0.0f, 0xFFFFFFFF, 1.0f, 0.0f },
   { -1.0f, -1.0f, -1.0f, 0.0f, -1.0f, 0.0f, 0xFFFFFFFF, 1.0f, 1.0f },
   {  1.0f, -1.0f, -1.0f, 0.0f, -1.0f, 0.0f, 0xFFFFFFFF, 0.0f, 1.0f },

   {  20.0f, 3.0f, -20.0f, 0.0f, -1.0f, 0.0f, 0xFFFFFFFF, 0.0f, 1.0f },  //sky
   {  20.0f, 3.0f,  20.0f, 0.0f, -1.0f, 0.0f, 0xFFFFFFFF, 0.0f, 0.0f },
   { -20.0f, 3.0f,  20.0f, 0.0f, -1.0f, 0.0f, 0xFFFFFFFF, 1.0f, 0.0f },
   { -20.0f, 3.0f,  20.0f, 0.0f, -1.0f, 0.0f, 0xFFFFFFFF, 1.0f, 0.0f },
   { -20.0f, 3.0f, -20.0f, 0.0f, -1.0f, 0.0f, 0xFFFFFFFF, 1.0f, 1.0f },
   {  20.0f, 3.0f, -20.0f, 0.0f, -1.0f, 0.0f, 0xFFFFFFFF, 0.0f, 1.0f },  

   { -20.0f, 3.0f, -20.0f, 1.0f, 0.0f, 0.0f, 0xFFFFFFFF, 0.0f, 0.0f }, //lwalls
   { -20.0f, 3.0f,  20.0f, 1.0f, 0.0f, 0.0f, 0xFFFFFFFF, 1.0f, 0.0f },
   { -20.0f,-3.0f, -20.0f, 1.0f, 0.0f, 0.0f, 0xFFFFFFFF, 0.0f, 1.0f },
   { -20.0f, 3.0f,  20.0f, 1.0f, 0.0f, 0.0f, 0xFFFFFFFF, 1.0f, 0.0f },
   { -20.0f,-3.0f, -20.0f, 1.0f, 0.0f, 0.0f, 0xFFFFFFFF, 0.0f, 1.0f },
   { -20.0f,-3.0f,  20.0f, 1.0f, 0.0f, 0.0f, 0xFFFFFFFF, 1.0f, 1.0f },

   { 20.0f, 3.0f, -20.0f, -1.0f, 0.0f, 0.0f, 0xFFFFFFFF, 1.0f, 0.0f }, //rwalls
   { 20.0f, 3.0f,  20.0f, -1.0f, 0.0f, 0.0f, 0xFFFFFFFF, 0.0f, 0.0f },
   { 20.0f,-3.0f, -20.0f, -1.0f, 0.0f, 0.0f, 0xFFFFFFFF, 1.0f, 1.0f },
   { 20.0f, 3.0f,  20.0f, -1.0f, 0.0f, 0.0f, 0xFFFFFFFF, 0.0f, 0.0f },
   { 20.0f,-3.0f, -20.0f, -1.0f, 0.0f, 0.0f, 0xFFFFFFFF, 1.0f, 1.0f },
   { 20.0f,-3.0f,  20.0f, -1.0f, 0.0f, 0.0f, 0xFFFFFFFF, 0.0f, 1.0f },

   { -20.0f, 3.0f,  20.0f, 0.0f, 0.0f, -1.0f, 0xFFFFFFFF, 0.0f, 0.0f }, //bwalls
   { -20.0f,-3.0f,  20.0f, 0.0f, 0.0f, -1.0f, 0xFFFFFFFF, 0.0f, 1.0f },
   {  20.0f, 3.0f,  20.0f, 0.0f, 0.0f, -1.0f, 0xFFFFFFFF, 1.0f, 0.0f },
   {  20.0f, 3.0f,  20.0f, 0.0f, 0.0f, -1.0f, 0xFFFFFFFF, 1.0f, 0.0f },
   {  20.0f,-3.0f,  20.0f, 0.0f, 0.0f, -1.0f, 0xFFFFFFFF, 1.0f, 1.0f },
   { -20.0f,-3.0f,  20.0f, 0.0f, 0.0f, -1.0f, 0xFFFFFFFF, 0.0f, 1.0f },

   { -20.0f, 3.0f,  -20.0f, 0.0f, 0.0f, 1.0f, 0xFFFFFFFF, 0.0f, 0.0f }, //fwalls
   { -20.0f,-3.0f,  -20.0f, 0.0f, 0.0f, 1.0f, 0xFFFFFFFF, 0.0f, 1.0f },
   {  20.0f, 3.0f,  -20.0f, 0.0f, 0.0f, 1.0f, 0xFFFFFFFF, 1.0f, 0.0f },
   {  20.0f, 3.0f,  -20.0f, 0.0f, 0.0f, 1.0f, 0xFFFFFFFF, 1.0f, 0.0f },
   {  20.0f,-3.0f,  -20.0f, 0.0f, 0.0f, 1.0f, 0xFFFFFFFF, 1.0f, 1.0f },
   { -20.0f,-3.0f,  -20.0f, 0.0f, 0.0f, 1.0f, 0xFFFFFFFF, 0.0f, 1.0f },

   {  20.0f, -3.0f, -20.0f, 0.0f, 1.0f, 0.0f, 0xFFFFFFFF, 0.0f, 1.0f },  //floor
   {  20.0f, -3.0f,  20.0f, 0.0f, 1.0f, 0.0f, 0xFFFFFFFF, 0.0f, 0.0f },
   { -20.0f, -3.0f,  20.0f, 0.0f, 1.0f, 0.0f, 0xFFFFFFFF, 1.0f, 0.0f },
   { -20.0f, -3.0f,  20.0f, 0.0f, 1.0f, 0.0f, 0xFFFFFFFF, 1.0f, 0.0f },
   { -20.0f, -3.0f, -20.0f, 0.0f, 1.0f, 0.0f, 0xFFFFFFFF, 1.0f, 1.0f },
   {  20.0f, -3.0f, -20.0f, 0.0f, 1.0f, 0.0f, 0xFFFFFFFF, 0.0f, 1.0f },

};
//Making sure to reference correctly sized vertex buffer
#define NUM_VERTICES (sizeof(app_vertices)/sizeof(my_vertex))

//Vertex buffers are a method of storing vertices to be rendered in an optimized manner.
IDirect3DVertexBuffer8 *app_vb=NULL;

float x_speed=0.0f;//Rotation spped on the axis
float y_speed=0.0f;
float z_pos=0.0f; //Used to change position of objects when rendering
float y_pos=-2.0f;
float x_pos=0.0f;

bool jump=false; //are we jumping?
float jump_count=0.0f; //used in jumping loop

float angle = 0.0f; //angle of turning with arrows
D3DXVECTOR3 lookat(0.0f, 0.0f, 0.0f); //vector for where we are looking
D3DXVECTOR3 pos(0.0f, 0.0f, -10.0f); //vector for the camera position
float turn = 0.02f; //turn speed
float velocity = 0.5f; //forward and back movement speed
const float pi2=D3DX_PI * 2.0f; //define what pi2 is

DWORD filter=0; //Variable to indicate our texture filter

BOOL lighting_on=TRUE;//Light on or off?
D3DLIGHT8 light; //create a light

// WinMain is the entry point of the app
int APIENTRY WinMain(HINSTANCE p_instance,HINSTANCE p_prev_instance,LPSTR p_cmd_line,int p_show){

   app_instance=p_instance;// Set global instance handle
   NOP(p_prev_instance,p_cmd_line,p_show);//Compiler gives warnings without this function

   ask_fullscreen();// Full screen, windowed or cancel?
   init_window();// Build our window to screen size choice

   init_d3d();//Build the D3D objects we need   
   init_scene();//Once prep of things required for rendering

   //This will loop until the app is terminated
   while(!app_done){
      message_pump(); //Check for window messages
      render(); //draw!
   }
   kill_scene(); 
   kill_d3d(); //clean D3D objects
   kill_window(); //...closes the window...
   return 0;
}


// This makes compiler thinks certain things are used which are not, eliminating warnings
void NOP(HINSTANCE p_prev_instance,LPSTR p_cmd_line,int p_show){
   p_prev_instance=p_prev_instance;
   p_cmd_line=p_cmd_line;
   p_show=p_show;
}

//Check for windows messages, if there are any dispatch to the handler
void message_pump(void){
MSG msg;
   if(PeekMessage(&msg, NULL, 0, 0,PM_REMOVE)){
      TranslateMessage(&msg);
      DispatchMessage(&msg);
   }
}
// Creat and setup D3D device according to user screen choice. 
void init_d3d(void){
HRESULT hr;
D3DPRESENT_PARAMETERS d3dpp;
D3DDISPLAYMODE display_mode;

   //In a D3D8 program you must create D3D8 first. 
   app_D3D = Direct3DCreate8( D3D_SDK_VERSION );
   if(!app_D3D ){
	   FatalError("Error Creating Direct3D");
   }

   //Get the (desktop) display mode. Only needed in window, not full screen
   hr=app_D3D->GetAdapterDisplayMode(D3DADAPTER_DEFAULT,&display_mode);
   if(FAILED(hr)){
      FatalError("Error while getting display mode\n");
   }

   //Clear out D3DPRESENT_PARAMETERS struct. Good idea to zero it out.
   ZeroMemory(&d3dpp,sizeof(d3dpp));

   //Same for full and windowed
   d3dpp.SwapEffect     = D3DSWAPEFFECT_DISCARD; // Get rid of previous frames
   d3dpp.hDeviceWindow  = app_main_window;  //This is our only/main (if more than one) window
   d3dpp.BackBufferCount= 1;  //Only 1 back buffer is needed

   //Z-buffer is needed for everything to render right
   d3dpp.EnableAutoDepthStencil = TRUE;
   d3dpp.AutoDepthStencilFormat = D3DFMT_D16;

   // BackBufferWidth and Height have to be set for full screen
   // these values are to determine the display mode.
   if(app_fullscreen){
      d3dpp.Windowed          = FALSE;
      d3dpp.BackBufferWidth   = app_width;
      d3dpp.BackBufferHeight  = app_height;
      d3dpp.BackBufferFormat  = find_16bit_mode(); //BBF is the pixel format, find pixel format
   }else{
      d3dpp.Windowed          = TRUE;
      d3dpp.BackBufferFormat  = display_mode.Format; //same format as desktop, GetAdapterDisplayMode()
   }

   hr=app_D3D->CreateDevice(D3DADAPTER_DEFAULT, //The default adapter, 2+ monitor can have > 1
                            D3DDEVTYPE_HAL, //Hardware acceleration instead of software renderer
                            app_main_window, // This is our window
                            D3DCREATE_SOFTWARE_VERTEXPROCESSING, //Processes vertices in software, 
										//slower than hard but works on all graphics cards
                            &d3dpp, //D3DPRESENT_PARAMETERS struct, tells what to build
                            &app_d3d_device); //will point to new device
   if(FAILED(hr)){
      FatalError("Error creating device\n");
   }
}

// One time preperation of our pyramid
void init_scene(void){
HRESULT hr;
unsigned char *vb_vertices;
D3DXMATRIX view_matrix;
D3DXMATRIX matProj;
	
   //D3DTSS_COLOROP is how the color of pixels is determined
   //D3DTOP_SELECTARG1 looks at D3DTSS_COLORARG1 to determine the color.
   //MODULATE determines color by COLORARG1 and COLORARG2
   app_d3d_device->SetTextureStageState(0,D3DTSS_COLOROP, D3DTOP_MODULATE);

   //D3DTSS_COLORARG1 is set to D3DTA_TEXTURE, taking the color completly from the 
   //texture, we could use D3DTA_DIFFUSE to use the color
   //CURRENT is vertex color if light off, or result of lighting if it is on
   app_d3d_device->SetTextureStageState(0,D3DTSS_COLORARG1, D3DTA_TEXTURE);
   app_d3d_device->SetTextureStageState(0,D3DTSS_COLORARG2, D3DTA_CURRENT);

   //Turn off alphablending/transparency to make it faster
   app_d3d_device->SetTextureStageState(0,D3DTSS_ALPHAOP, D3DTOP_DISABLE);
   //turn off culling to see room everywhere
   app_d3d_device->SetRenderState(D3DRS_CULLMODE,D3DCULL_NONE);
   //MAGFILTER is how the texture is drawn when it is bigger than the real size
   //MINFILTER is when it is drawn smaller. 
   //D3DTEXF_LINEAR & D3DTEXF_POINT. Linear is smoother but more expensive.
   app_d3d_device->SetTextureStageState(0,D3DTSS_MAGFILTER, D3DTEXF_NONE);
   app_d3d_device->SetTextureStageState(0,D3DTSS_MINFILTER, D3DTEXF_NONE);

   //Zero the light struct and fill in the fields needed
   ZeroMemory( &light, sizeof(D3DLIGHT8) );
   light.Type=D3DLIGHT_POINT;//Point light, sinces in all directions
   light.Diffuse.r=2.0f; //Light color
   light.Diffuse.g=2.0f;
   light.Diffuse.b=2.0f;
   light.Range=1000.0f;//Range for illumination
   light.Position=D3DXVECTOR3(0.0f,2.0f,-10.0f); //position of the light
   light.Attenuation0=1.0f;//How light falls off, set to full illumination, no fall off
   
   app_d3d_device->SetLight( 0, &light );//set our light to an index
   app_d3d_device->LightEnable( 0, TRUE); //Enable our light, however this doesn't mean the light is on.

   //Turn off D3D lighting //ambient is default lighting for objects not hit by our light
   app_d3d_device->SetRenderState( D3DRS_LIGHTING, TRUE );
   app_d3d_device->SetRenderState(D3DRS_AMBIENT,0x00050505);

   // Turn on the zbuffer.  Zbuffer draws things in the proper order (overlapping items
   // appear how they should, whatever is in front you see.)
   app_d3d_device->SetRenderState( D3DRS_ZENABLE, TRUE );

   //Vertex buffer large enough for vertices. Tell D3D it is write-only and allow
   //it to determine where in memory it goes
   hr=app_d3d_device->CreateVertexBuffer(NUM_VERTICES*sizeof(my_vertex), //Size of memory to be allocated
                                       D3DUSAGE_WRITEONLY, // WRITEONLY is faster
                                       D3D8T_CUSTOMVERTEX, //Our cust vertex with coordinates and color
                                       D3DPOOL_MANAGED,   //DX can manage the memory
                                       &app_vb);          //Pointer to our Vertex Buffer
   if(FAILED(hr)){
      FatalError("Error creating vertex buffer");
   }

   //load our textures
   hr=D3DXCreateTextureFromFile( app_d3d_device, "text.bmp",&app_texture);
   if(FAILED(hr)){
      FatalError("Error loading texture");
   }
   hr=D3DXCreateTextureFromFile( app_d3d_device, "sky.bmp",&sky_texture);
   if(FAILED(hr)){
      FatalError("Error loading texture");
   }
   hr=D3DXCreateTextureFromFile( app_d3d_device, "brick.bmp",&wall_texture);
   if(FAILED(hr)){
      FatalError("Error loading texture");
   }
   hr=D3DXCreateTextureFromFile( app_d3d_device, "floor.bmp",&floor_texture);
   if(FAILED(hr)){
      FatalError("Error loading texture");
   }

   //call the model loading function to load our cube
   load_mesh(app_d3d_device, "box.x");

   //Lock the Vertex Buffer so Direct3D knows it is being modified, then copy
   //data and Unlock it so Direct3D knows we're done.
   hr=app_vb->Lock(0, //Offset, start at the beginning
                 0, //SizeToLock, 0 = lock whole thing
                 &vb_vertices, //This will point to what is in the VB
                 0);  //flags
   if(FAILED(hr)){
      FatalError("Error Locking buffer");
   }
   
   //vb_vertices points to vertices inside the Vertex buffer, to fill in our VB
   //we copy to vb_vertices.
   memcpy(vb_vertices, app_vertices, sizeof(app_vertices) );

   app_vb->Unlock();//Unlock, D3D knows we are done now

   //Build View Matrix...(camera), viewpoint, looking to, what is up
   D3DXMatrixLookAtLH(&view_matrix,&pos,
                                   &lookat,
                                   &D3DXVECTOR3( 0.0f, 1.0f, 0.0f ));
   //We set the camera, but will be altering it later
   app_d3d_device->SetTransform(D3DTS_VIEW,&view_matrix);

   D3DXMatrixPerspectiveFovLH(&matProj, //Resulting Matrix
                              D3DX_PI/4,//Field of View, in radians. (PI/4 = typical)
                              ((float)app_width / (float)app_height), //Aspect ratio
                              1.0f,      //Near view plane
                              100.0f ); // Far view plane

   //Projection matrix
   app_d3d_device->SetTransform( D3DTS_PROJECTION, &matProj );

   //initialize directinput
   DirectInput8Create(app_instance, DIRECTINPUT_VERSION,
          IID_IDirectInput8, (void**)&app_DirectInput, NULL);

   app_DirectInput->CreateDevice(GUID_SysKeyboard, &app_Keyboard, NULL); //create keyboard
   app_Keyboard->SetDataFormat(&c_dfDIKeyboard); //set data format for kboard

   //set the keyboard to work when app is in focus and does not interfere with other 
   //apps also using this device
   app_Keyboard->SetCooperativeLevel(app_main_window, DISCL_FOREGROUND | DISCL_NONEXCLUSIVE);
   
   app_Keyboard->Acquire(); //we can now access this device
}

// Test 16-bit modes to find supported.
D3DFORMAT find_16bit_mode(void){
HRESULT hr;

   //Test for R5G6B5.  All 16-bits are used, full 64K of color
   hr=app_D3D->CheckDeviceType(D3DADAPTER_DEFAULT,D3DDEVTYPE_HAL,D3DFMT_R5G6B5,D3DFMT_R5G6B5,FALSE);
   if(SUCCEEDED(hr)){
      OutputDebugString("D3DFMT_R5G6B5\n");
      return D3DFMT_R5G6B5;
   }

   //Test X1R5G5B5. Technically 15-bit and only 32K colours
   hr=app_D3D->CheckDeviceType(D3DADAPTER_DEFAULT,D3DDEVTYPE_HAL,D3DFMT_X1R5G5B5,D3DFMT_X1R5G5B5,FALSE);
   if(SUCCEEDED(hr)){
      OutputDebugString("D3DFMT_X1R5G5B5\n");
      return D3DFMT_X1R5G5B5;
   }

   //Graphics card doesn't support either for some reason
   FatalError("Couldn't find a supported mode\n");

   return (D3DFORMAT)NULL;
}

// Check to see if the user wants full screen, windowed, or to cancel
void ask_fullscreen(void){
int full_result;

   full_result=MessageBox(NULL,"Would you like to run in fullscreen mode?",app_name,
                          MB_YESNOCANCEL|MB_ICONQUESTION);
   switch(full_result){
      case IDCANCEL:
         OutputDebugString("User cancelled program");
         MessageBox(NULL,"User cancelled program",app_name,MB_OK);
         exit(5);
         break;
      case IDNO:
         app_fullscreen=false;
         break;
      case IDYES:
         app_fullscreen=true;
         break;
      case 0:
         OutputDebugString("MessageBox would not open.");
         exit(10);
         break;
   }
}

// Clears screen, draw pyramid, display
void render(void){
D3DXMATRIX matWorld;
D3DXMATRIX rot_matrix;     //rotation matrix
D3DXMATRIX rot_matrix1;     //rotation matrix
D3DXMATRIX rot_matrix2;     //rotation matrix
D3DXMATRIX trans_matrix;   //translation matrix
D3DXMATRIX view_matrix, blah;
static float rot_x=0; //rotation for our triangle
static float rot_y=0;

keyinput(); //read the keyboard to see what buttons are pressed

   rot_y+=0.004f;

   D3DXMatrixLookAtLH(&view_matrix,
                         &pos,
                         &lookat,
                         &D3DXVECTOR3( 0.0f, 1.0f, 0.0f ));

   app_d3d_device->SetTransform(D3DTS_VIEW,&view_matrix);

   if(jump){
	   if(jump_count<25)
			y_pos+=0.05f;
	   if(jump_count>=25)
			y_pos-=0.05f;
	   jump_count++;
	   if(jump_count==50){
		   jump=false;
		   jump_count=0;
	   }
   }

   //clear buffer to color and clear zbuffer
   app_d3d_device->Clear( 0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER , D3DCOLOR_XRGB(0,0,0), 1.0f, 0 );

   //tell device we are ready to draw/render
   if(SUCCEEDED(app_d3d_device->BeginScene())){

      //Tell D3D we are using plain vertex format
      app_d3d_device->SetVertexShader(D3D8T_CUSTOMVERTEX);

      //Vertex buffer we created is the stream to read from
      app_d3d_device->SetStreamSource(0,app_vb,sizeof(my_vertex));

	  draw(0,0,0,0,0,sky_texture, D3DPT_TRIANGLELIST, 12, 2); //draw sky
	  draw(0,0,0,0,0,wall_texture, D3DPT_TRIANGLELIST, 18, 8); //draw walls
	  draw(0,0,0,0,0,floor_texture, D3DPT_TRIANGLELIST, 42, 2); //draw floor


	  //draw pyramid
	  app_d3d_device->SetTexture( 0, app_texture ); //look at draw for comments
	  D3DXMatrixRotationY(&rot_matrix2,rot_y);
      D3DXMatrixTranslation(&trans_matrix,18,0,18);
      D3DXMatrixMultiply(&matWorld,&rot_matrix2,&trans_matrix); 
      app_d3d_device->SetTransform(D3DTS_WORLD,&matWorld );

      app_d3d_device->DrawPrimitive(D3DPT_TRIANGLEFAN,0,4);
	  app_d3d_device->DrawPrimitive(D3DPT_TRIANGLELIST, 6, 2);

	  //setup matrix for the mesh
      D3DXMatrixRotationX(&rot_matrix1,0);
	  D3DXMatrixRotationY(&rot_matrix2,0);
	  D3DXMatrixMultiply(&rot_matrix, &rot_matrix1, &rot_matrix2);
      D3DXMatrixTranslation(&trans_matrix, x_pos, y_pos+.3f, z_pos);
      D3DXMatrixMultiply(&matWorld,&rot_matrix,&trans_matrix); 

      //set the world matrix
      app_d3d_device->SetTransform(D3DTS_WORLD,&matWorld );

	  //draw mesh (cube)
	  for(DWORD i = 0; i < NumMaterials; i++)
        {
            app_d3d_device->SetMaterial(&MeshMaterials[i]);
            app_d3d_device->SetTexture(0, MeshTextures[i]);
        
            mesh->DrawSubset(i);
        }

      //Tell device this frame is done being rendered
      app_d3d_device->EndScene();
   }
   //Display what we did
   app_d3d_device->Present( NULL, NULL, NULL, NULL );
   //good idea to set texture to NULL so your textures can be released properly
   app_d3d_device->SetTexture(0,NULL);
}
// Register window class and create window
void init_window(void){
ULONG window_width, window_height;
WNDCLASS window_class;
DWORD style;

   //WNDCLASS structure.  Window classes = template for win creation
   //can have many windows from one class
   window_class.style          = CS_OWNDC;
   window_class.cbClsExtra     = 0;
   window_class.cbWndExtra     = 0;
   window_class.hInstance      = app_instance;
   window_class.hIcon          = LoadIcon(NULL,IDI_APPLICATION);
   window_class.hCursor        = LoadCursor(NULL,IDC_ARROW);
   window_class.hbrBackground  = (HBRUSH)GetStockObject(BLACK_BRUSH);
   window_class.lpszMenuName   = NULL;
   window_class.lpszClassName  = "DH Class";
   //default window handler, all windows messages sent to this function.
   window_class.lpfnWndProc    = default_window_proc;

   //Register the class
   if(!RegisterClass(&window_class)){
      FatalError("Error registering window class");
   }

   //Cover desktop with window if in full screen
   //Not needed, but smoother transition
   if(app_fullscreen){
      window_width=GetSystemMetrics(SM_CXSCREEN);
      window_height=GetSystemMetrics(SM_CYSCREEN);
      style=WS_POPUP;
   }else{ //otherwise give the window dimensions
      window_width=app_width;
      window_height=app_height;
      style=WS_OVERLAPPED|WS_SYSMENU;
   }

   app_main_window=CreateWindow("DH Class",app_name,style,
                              0,0,window_width,window_height,NULL,NULL,
                              app_instance,NULL);

   if(!app_main_window){
      FatalError("Error creating window");
   }

   //Makes sure the window is visible and has focus
   ShowWindow(app_main_window,SW_SHOW);
   UpdateWindow(app_main_window);
   SetFocus(app_main_window);

}
// Release all D3D resources in opposite order of what we created
// Test for NULL since all was initialized to NULL, meaning don't release
// something that wasn't even created
void kill_d3d(void){

   if(app_d3d_device){
      app_d3d_device->Release();
      app_d3d_device=NULL;
   }

   if(app_D3D){
      app_D3D->Release();
      app_D3D=NULL;
   }
}
// Clean up objects we needed for rendering
void kill_scene(void){

   if(app_vb){
      app_vb->Release();
      app_vb=NULL;
   }
   if(app_texture){
      app_texture->Release();
      app_texture=NULL;
   }

   if( MeshMaterials )
    delete[] MeshMaterials;

	if( MeshTextures )
	{
		for( DWORD i = 0; i < NumMaterials; i++ )
		{
		 if( MeshTextures[i] )
		        MeshTextures[i]->Release();
	   }
	delete[] MeshTextures;
	}

	if( mesh )
		mesh->Release();

	 if(app_Keyboard)
    {
        app_Keyboard->Unacquire();
    }
}
// Close window, clean messages and unregister
//he standard way involves putting the clean-up code in your window handler
void kill_window(void){
  
   if(app_main_window){ //is the window valid
      if(!DestroyWindow(app_main_window)){ //if destroying window fails
         OutputDebugString("Failed to DestroyWindow\n");
         MessageBox(NULL,"Destroy Window Failed",app_name,MB_OK|MB_ICONERROR|MB_TOPMOST);
      }else{
         MSG msg;
         while(PeekMessage(&msg, NULL, 0, 0,PM_REMOVE)){ //clean pending messages
            DispatchMessage(&msg);
         }
      }
      app_main_window=NULL; //set window handle to NULL, to be safe
   }

   //Unregister the window. If you have more than one window with the class
   //you need to close them all before unregistering
   if(!UnregisterClass("DH Class",app_instance)){
      OutputDebugString("Failed to Unregister Window\n");
      MessageBox(NULL,"Unregister Failed",app_name,MB_OK|MB_ICONERROR|MB_TOPMOST);
   }

}
// Close all resources, tell user and quit
void FatalError(const char *error_msg){

   kill_scene();
   kill_d3d();
   kill_window();
   OutputDebugString( error_msg );
   OutputDebugString("\n");
   MessageBox(NULL, error_msg,app_name, MB_OK );
   exit(5);
}
// Handle windows messages, unhandles messages go to DefwWindowProc() (part of Win32 API)
LRESULT CALLBACK default_window_proc(HWND hwnd,UINT msg,WPARAM wparam,LPARAM lparam){
HDC hdc;
PAINTSTRUCT ps;
   
int virt_key = (int) wparam;
   switch(msg){
      case WM_KEYDOWN:  // If a key is pressed, end the application
         switch(virt_key){ 
            case 'L':
               lighting_on=!lighting_on;
               app_d3d_device->SetRenderState(D3DRS_LIGHTING,lighting_on);
               break;
            default:
               //app_done=true;
               break;
         }
         return 0;
         return 0;
      case WM_PAINT:    //Repaint the window
         hdc=BeginPaint(hwnd,&ps);
         EndPaint(hwnd,&ps);
         return 0;
      case WM_CLOSE:    // If close button is hit, end the application
         app_done=true;
         return 0;
      case WM_DESTROY:  //If the window is being destroyed, tell Windows
         PostQuitMessage(0);
         return 0;
   }
   return (DefWindowProc(hwnd,msg,wparam,lparam));
}

//directinput function to get keystrokes
void keyinput(void)
{
	char KeyboardState[256]; 
	app_Keyboard->GetDeviceState(sizeof(KeyboardState),(LPVOID)&KeyboardState);
	if(KEYDOWN(KeyboardState, DIK_SPACE)) //Hitting space causes the cube to jump
		{
			if(!jump){
				jump=true;
            }
		}
	if(KEYDOWN(KeyboardState, DIK_ESCAPE)) //quit program
		{
				app_done=true;
		}
	if(KEYDOWN(KeyboardState, DIK_UP)) //this calculates the new x and z positions
		{							   //for the camera and the cube
			pos.x+=(velocity) * (sinf(angle));
			pos.z+=(velocity) * (cosf(angle));
			x_pos+=(velocity) * (sinf(angle));
			z_pos+=(velocity) * (cosf(angle));
		}
	if(KEYDOWN(KeyboardState, DIK_DOWN)) //same as above
		{
			pos.x-=(velocity) * (sinf(angle));
			pos.z-=(velocity) * (cosf(angle));
			x_pos-=(velocity) * (sinf(angle));
			z_pos-=(velocity) * (cosf(angle));
		}
	if(KEYDOWN(KeyboardState, DIK_LEFT)) //this alters the angle to generate a new lookat point
		{
			angle-=turn;
			if(angle < 0){
				angle+=pi2;
			}
		}
	if(KEYDOWN(KeyboardState, DIK_RIGHT)) //same as above
		{
			angle+=turn;
			if(angle > pi2){
				angle-=pi2;
			}
		}
	if(KEYDOWN(KeyboardState, DIK_W)) //move the cube ONLY forward (in relation to camera)
		{
			x_pos+=(velocity) * (sinf(angle));
			z_pos+=(velocity) * (cosf(angle));
		}
	if(KEYDOWN(KeyboardState, DIK_S)) //move the cube ONLY backwards (in relation to cam)
		{
			x_pos-=(velocity) * (sinf(angle));
			z_pos-=(velocity) * (cosf(angle));
		}
	if(KEYDOWN(KeyboardState, DIK_D)) //towards negative on x axis
		{
			x_pos-=.06f;
		}
	if(KEYDOWN(KeyboardState, DIK_G)) //towards pos on x axis
		{
			x_pos+=.06f;
		}
	if(KEYDOWN(KeyboardState, DIK_F)) //towards negative on z axis
		{
			z_pos-=.06f;
		}
	if(KEYDOWN(KeyboardState, DIK_R)) //towards pos on z axis
		{
			z_pos+=.06f;
		}

   lookat.x=sinf(angle)+pos.x;  //this always makes sure the new lookat is calculated
   lookat.y=pos.y;				//if a button is pressed
   lookat.z=cosf(angle)+pos.z;
}

//This function loads in a .x file and then sorts out all the mesh stuff
void load_mesh(LPDIRECT3DDEVICE8 meshdevice, LPSTR meshName)
{
	// Load the mesh from the specified file.
	D3DXLoadMeshFromX(meshName, D3DXMESH_SYSTEMMEM, 
                               meshdevice, NULL, 
                               &MaterialBuffer, &NumMaterials, 
                               &mesh);

	D3DXMATERIAL* d3dxMaterials = (D3DXMATERIAL*)MaterialBuffer->GetBufferPointer();
	MeshMaterials = new D3DMATERIAL8[NumMaterials]; //array for mesh materials
    MeshTextures  = new LPDIRECT3DTEXTURE8[NumMaterials]; //array for mesh textures

	for(DWORD i = 0; i <NumMaterials; i++)
    {
	    MeshMaterials[i] = d3dxMaterials[i].MatD3D; //copy the mesh materials
        MeshMaterials[i].Ambient = MeshMaterials[i].Diffuse; //set ambient color
        
		// Create the texture for the mesh
		if( FAILED( D3DXCreateTextureFromFile( meshdevice, 
                                       d3dxMaterials[i].pTextureFilename, 
                                       &MeshTextures[i] ) ) )
		MeshTextures[i] = NULL;
	}

	D3DXComputeNormals(mesh, NULL); //compute the normals so lighting looks right

	MaterialBuffer->Release();//release the materialbuffer and return
	return;
}

void draw(float rotx, float roty, float transx, float transy, float transz, LPDIRECT3DTEXTURE8 textName,
		  D3DPRIMITIVETYPE drawType, int startVertice, int numVertice){

	  D3DXMATRIX matWorld;  //create a matrix for rotations, translations and the world
	  D3DXMATRIX rot_matrixX;     
	  D3DXMATRIX rot_matrixY, rot_matrix;     
	  D3DXMATRIX trans_matrix;
	  static float rot_y=0; //rotation variables, this function is not processing
	  static float rot_x=0; //any rotations yet
      rot_y+=roty;
	  rot_x+=rotx;

	  app_d3d_device->SetTexture( 0, textName ); //this gets the current texture to use
	  D3DXMatrixRotationX(&rot_matrixX,rot_x); //does rotation on X axis
	  D3DXMatrixRotationY(&rot_matrixY,rot_y); //does rotation on Y axis
	  D3DXMatrixTranslation(&trans_matrix,transx,transy,transz); //Translates on 3 axis
	  D3DXMatrixMultiply(&rot_matrix, &rot_matrixX, &rot_matrixY); //multiply the matrixes
	  D3DXMatrixMultiply(&matWorld, &rot_matrix, &trans_matrix); //together to get one
	  app_d3d_device->SetTransform(D3DTS_WORLD,&matWorld ); //set the world matrix
	  app_d3d_device->DrawPrimitive(drawType, startVertice, numVertice); //draw our vertices
}